#ifndef MAT3X3_HH
#define MAT3X3_HH

/*
  James R. Diebel
  Stanford University
  
  Started: 3 December 2004

  vec3d.hh - Class definition of basic 3d vector class

  Depends on Vec3d class (vec3d.hh)
*/

#include <math.h>
#include <stdlib.h>
#include <iostream>
#include "vec3d.hh"
#ifndef PI
#define PI 3.14159265358979323846
#endif


using namespace std;

namespace bmtk {

// little support class used to facilitate efficient [] referencing
class Row1x3 {
public:
  float* x;

  float operator [] (const int index) const;
  float& operator [] (const int index);
};

class Mat3x3 {
public: // everything is public

  // data is a 1D array of size 9
  float x[9];
  Row1x3 r[3];

  // Constructors
  Mat3x3();
  Mat3x3(bool inf);
  Mat3x3(float x11, float x12, float x13,
	 float x21, float x22, float x23,
	 float x31, float x32, float x33);
  Mat3x3(const float a);
  Mat3x3(const float* x);
  Mat3x3(const Vec3d& v);
  Mat3x3(const Vec3d& v0, const Vec3d& v1);
  Mat3x3(const Vec3d& v0, const Vec3d& v1, const Vec3d& v2);
  Mat3x3(const Vec3d& v0, const Vec3d& v1, const Vec3d& v2,
	 const Vec3d& lambda);
    

  // Assignments
  Mat3x3 operator = (const Mat3x3& m);
  
  // Reference operator: zero-based square bracket referencing
  Row1x3 operator [] (const int index) const;
  Row1x3& operator [] (const int index);  

  // Addition and subtraction: <matrix> +/- <matrix>
  Mat3x3 operator + (const Mat3x3& m) const;
  void operator += (const Mat3x3& m);
  Mat3x3 operator - (const Mat3x3& m) const;
  void operator -= (const Mat3x3& m);

  // Multiplication and division: <matrix> *// <scalar>
  Mat3x3 operator * (const float a) const;
  void operator *= (const float a);
  Mat3x3 operator / (const float a) const;
  void operator /= (const float a);

  // Multiplication: <matrix> * <vector>
  Vec3d operator * (const Vec3d& v) const;
  
  // Multiplication: <matrix> * <vector>
  Mat3x3 operator * (const Mat3x3& m) const;
  void operator *= (const Mat3x3& m);

  // Determinant, inverse, transpose, etc.
  float det() const;
  Mat3x3 inv() const;
  void transIt();
  Mat3x3 trans() const;
  static Mat3x3 outer(const Vec3d& v0, const Vec3d& v1);
  static Mat3x3 eigenReconst(const Vec3d& v0, const Vec3d& v1, 
			     const Vec3d& v2, const Vec3d& l);
    // output functions
  void print();
};

// Define a few global operators to handle cases where 
// a Mat3x3 is the second argument in a binary operator
Mat3x3 operator * (const float a, const Mat3x3 &m);
Vec3d operator * (const Vec3d& v, const Mat3x3 &m);
void operator *= (Vec3d& v, const Mat3x3& m);

// Define << operator to handle output to screen
ostream& operator << (ostream& os, const Mat3x3 &m);

} // namespace bmtk

#endif
