#ifndef USPLINE_HH
#define USPLINE_HH
/*
  James R. Diebel
  Stanford University
  
  Started: 22 November 2004
  Last revised: 2004

  uspline.hh - defines class for cubic spline that approximates
  a single-valued function y(x) where our samples over x must be
  uniform - thus the 'u' in the 'uspline'

  Depends on:
  - Matrix inversion routine (matrix.hh)
*/

#include "matrix.hh"

namespace bmtk {

class USpline {
public:
  // Data
  int n; // number of control points
  float **A; // matrix used for coefficient generation
  float* y; // vector of control points
  float* a; // vector of coefficients, also the y-control points
  float* b; 
  float* c;
  float* d; // vector of higher-order coefficients
  float b0,bn; // slopes at endpoints
  float xmin,xmax; // range of independent coordinate
  float span,interval,dpdx;
  
  // Methods
  USpline(int n_);
  USpline(int n_, float xmin_, float xmax_, float b0_, float bn_);
  USpline(float* y_, int n_);
  ~USpline();
  
  // generate spline coefficients, evaluate value of spline
  void update();
  float evalf(float x);
  float evaldf(float x);
  void print();
};

} // namespace bmtk

#endif
