/*
  James R. Diebel
  Stanford University

  Started: 24 August 2004

  ply2msh.cc - Utility to convert a PLY file to a mesh (MSH) file

  Depends on:
  - Mesh class (mesh.hh)
  -- Vec3d class (vec3d.hh)
  -- Vert class (vert.hh)trimesh
  -- Edge class (edge.hh)
  -- Face class (face.hh)
  - Timer class (timer.hh)
  - TriMesh class (trimesh.h)
*/


#include <iostream>
#include "bmtk/mesh.hh"
#include "bmtk/timer.hh"
#include <trimesh/trimesh.h>

using namespace std;
using namespace bmtk;

int main (int argc, char *argv[]) {
  // if bad command line args, give usage instructions and exit
  if (argc < 3) {
    cout << endl
	 << "Usage: ply2msh infile.ply outfile.msh" << endl
	 << "    infile.ply must be a little-endian binary ply file" << endl
	 << "    outfile.msh is the name of the output msh file" << endl;
    exit(1);
  }

  // create empty Mesh and then fill it with data from ply file
  Mesh m;
  m.buildFromPLY(argv[1]);

  // output it to the specified file
  m.writeMesh(argv[2]);

  return 0;
}


