/*
  James R. Diebel
  Stanford University

  Started: 20 January 2005

  segmsh.cc - utility to segment a mesh and output a region file

  Depends on:
  - Mesh class (mesh.hh)
  -- Vec3d class (vec3d.hh)
  -- Vert class (vert.hh)
  -- Edge class (edge.hh)
  -- Face class (face.hh)
  - Timer class (timer.hh)
  - TriMesh class (trimesh.h)
*/


#include <iostream>
#include "bmtk/mesh.hh"
#include "bmtk/timer.hh"
#include <trimesh/trimesh.h>

using namespace std;
using namespace bmtk;

int main (int argc, char *argv[]) {
  // if bad command line args, give usage instructions and exit
  if (argc < 3) {
    cout << endl
	 << "Usage: segmsh infile.msh outfile.reg" << endl
	 << "    infile.msh must be an msh file" << endl
	 << "    outfile.reg is a region file" << endl;
    exit(1);
  }

  // read Mesh from file
  Mesh m;
  m.readMesh(argv[1]);
  m.readOptions("mrf_options.dat");

  // segment the mesh
  m.findRegionProps();
  m.runSegmentation();

  // output the segmentation
  m.exportRegions(argv[2]);

  return 0;
}
